import os
from typing import List
from dataclasses import dataclass

from .korpora import Korpus, LabeledSentencePairKorpusData, LabeledSentencePair
from .utils import fetch, default_korpora_path, load_text


KORSTS_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorSTS/sts-train.tsv',
        'destination': 'korsts/sts-train.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorSTS/sts-dev.tsv',
        'destination': 'korsts/sts-dev.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorSTS/sts-test.tsv',
        'destination': 'korsts/sts-test.tsv',
        'method': 'download'
    },
]

description = """    Author : KakaoBrain
    Repository : https://github.com/kakaobrain/KorNLUDatasets
    References :
        - Ham, J., Choe, Y. J., Park, K., Choi, I., & Soh, H. (2020). KorNLI and KorSTS: New Benchmark
           Datasets for Korean Natural Language Understanding. arXiv preprint arXiv:2004.03289.
           (https://arxiv.org/abs/2004.03289)

    This is the dataset repository for our paper
    "KorNLI and KorSTS: New Benchmark Datasets for Korean Natural Language Understanding."
    (https://arxiv.org/abs/2004.03289)
    We introduce KorNLI and KorSTS, which are NLI and STS datasets in Korean."""

license = """    Creative Commons Attribution-ShareAlike license (CC BY-SA 4.0)
    Details in https://creativecommons.org/licenses/by-sa/4.0/"""


@dataclass
class KorSTSExample(LabeledSentencePair):
    genre: str
    filename: str
    year: str


class KorSTSData(LabeledSentencePairKorpusData):
    genres: List[str]
    filenames: List[str]
    years: List[str]

    def __init__(self, dataname, texts, pairs, labels, genres, filenames, years):
        super().__init__(dataname, texts, pairs, labels)
        if not (len(labels) == len(genres) == len(filenames) == len(years)):
            raise ValueError('All length of `texts`, `pairs`, `labels`, `genres`, `filenames`, `years` should be same')
        self.genres = genres
        self.filenames = filenames
        self.years = years

    def __getitem__(self, index):
        return KorSTSExample(
            text=self.texts[index],
            pair=self.pairs[index],
            label=float(self.labels[index]),
            genre=self.genres[index],
            filename=self.filenames[index],
            year=self.years[index],
        )

    def get_all_genres(self):
        return self.genres

    def get_all_filenames(self):
        return self.filenames

    def get_all_years(self):
        return self.years


class KorSTSKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_korsts(root_dir, force_download)

        for info in KORSTS_FETCH_INFORMATION:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
            returns = self.cleaning(load_text(local_path, num_heads=1))
            texts, pairs, labels, genres, filenames, years = returns
            if 'train' in info['destination']:
                self.train = KorSTSData(
                    'KorSTS.train', texts, pairs, labels, genres, filenames, years)
            elif 'dev' in info['destination']:
                self.dev = KorSTSData(
                    'KorSTS.dev', texts, pairs, labels, genres, filenames, years)
            elif 'test' in info['destination']:
                self.test = KorSTSData(
                    'KorSTS.test', texts, pairs, labels, genres, filenames, years)
            else:
                raise ValueError('Check `KORSTS_FETCH_INFORMATION`')

    def cleaning(self, raw_lines: List[str]):
        separated_lines = [line.split('\t') for line in raw_lines]
        for i_sent, separated_line in enumerate(separated_lines):
            if len(separated_line) != 7:
                raise ValueError(f'Found some errors in line {i_sent}: {separated_line}')
        genres, filenames, years, _, labels, texts, pairs = zip(*separated_lines)
        return texts, pairs, labels, genres, filenames, years

    def get_all_texts(self):
        return self.train.get_all_texts() + self.dev.get_all_texts() + self.test.get_all_texts()

    def get_all_pairs(self):
        return self.train.get_all_pairs() + self.dev.get_all_pairs() + self.test.get_all_pairs()

    def get_all_labels(self):
        return self.train.get_all_labels() + self.dev.get_all_labels() + self.test.get_all_labels()

    def get_all_genres(self):
        return self.train.get_all_genres() + self.dev.get_all_genres() + self.test.get_all_genres()

    def get_all_filenames(self):
        return self.train.get_all_filenames() + self.dev.get_all_filenames() + self.test.get_all_filenames()

    def get_all_years(self):
        return self.train.get_all_years() + self.dev.get_all_years() + self.test.get_all_years()


def fetch_korsts(root_dir, force_download):
    for info in KORSTS_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'korsts', force_download)



import os
from typing import List

from .korpora import Korpus, LabeledSentencePairKorpusData, LabeledSentenceKorpusData, SentencePairKorpusData
from .utils import load_text


class CustomLabeledSentencePairKorpus(Korpus):
    """
    We assume that all custom LabeledSentencePair files are in `tap-separated text` format

        sent a\tsent b\tlabel_ab
        sent c\tsent d\tlabel_cd

    Examples:
        If custom corpus text files have no header

            >>> paths = [
            >>>     'path/to/mycorpus/train.tsv',
            >>>     'path/to/mycorpus/test.tsv',
            >>> ]
            >>> custom_korpus = CustomLabeledSentencePairKorpus(paths)

        If these files have headers

            >>> paths = [
            >>>     'path/to/mycorpus/train.tsv',
            >>>     'path/to/mycorpus/test.tsv',
            >>> ]
            >>> custom_korpus = CustomLabeledSentencePairKorpus(
            >>>     paths, num_headers=num_headers)
    """
    def __init__(self, paths, num_headers=0):
        super().__init__()
        for path in paths:
            attribute_name = get_attribute_name(path)
            raw_documents = load_text(path, num_heads=num_headers)
            texts, pairs, labels = self.cleaning(raw_documents)
            setattr(self, attribute_name, LabeledSentencePairKorpusData(
                'custom LabeledSentencePairKorpus', texts, pairs, labels))

    def cleaning(self, raw_documents: List[str], **kargs):
        examples = [example.split('\t') for example in raw_documents]
        for i_sent, columns in enumerate(examples):
            if len(columns) != 3:
                print(columns)
                raise ValueError(f'Found some errors in line {i_sent}: {examples[i_sent]}')
        texts, pairs, labels = zip(*examples)
        return texts, pairs, labels


class CustomLabeledSentenceKorpus(Korpus):
    """
    We assume that all custom LabeledSentence files are in `tap-separated text` format

        sent 1\tlabel_1
        sent 2\tlabel_2

    Examples:
        If custom corpus text files have no header

            >>> paths = [
            >>>     'path/to/mycorpus/train.tsv',
            >>>     'path/to/mycorpus/test.tsv',
            >>> ]
            >>> custom_korpus = CustomLabeledSentenceKorpus(paths)

        If these files have headers

            >>> paths = [
            >>>     'path/to/mycorpus/train.tsv',
            >>>     'path/to/mycorpus/test.tsv',
            >>> ]
            >>> custom_korpus = CustomLabeledSentenceKorpus(
            >>>     paths, num_headers=num_headers)
    """
    def __init__(self, paths, num_headers=0):
        super().__init__()
        for path in paths:
            attribute_name = get_attribute_name(path)
            raw_documents = load_text(path, num_heads=num_headers)
            texts, labels = self.cleaning(raw_documents)
            setattr(self, attribute_name, LabeledSentenceKorpusData(
                'custom LabeledSentenceKorpus', texts, labels))

    def cleaning(self, raw_documents: List[str], **kargs):
        examples = [example.split('\t') for example in raw_documents]
        for i_sent, columns in enumerate(examples):
            if len(columns) != 2:
                print(columns)
                raise ValueError(f'Found some errors in line {i_sent}: {examples[i_sent]}')
        texts, labels = zip(*examples)
        return texts, labels


class CustomSentencePairKorpus(Korpus):
    """
    We assume that all custom SentencePair files are in `tap-separated text` format

        sent a\tsent b
        sent c\tsent d

    Examples:
        If custom corpus text files have no header

            >>> paths = [
            >>>     'path/to/mycorpus/train.tsv',
            >>>     'path/to/mycorpus/test.tsv',
            >>> ]
            >>> custom_korpus = CustomSentencePairKorpus(paths)

        If these files have headers

            >>> paths = [
            >>>     'path/to/mycorpus/train.tsv',
            >>>     'path/to/mycorpus/test.tsv',
            >>> ]
            >>> custom_korpus = CustomSentencePairKorpus(
            >>>     paths, num_headers=num_headers)
    """
    def __init__(self, paths, num_headers=0):
        super().__init__()
        for path in paths:
            attribute_name = get_attribute_name(path)
            raw_documents = load_text(path, num_heads=num_headers)
            texts, pairs = self.cleaning(raw_documents)
            setattr(self, attribute_name, SentencePairKorpusData(
                'custom SentencePairKorpus', texts, pairs))

    def cleaning(self, raw_documents: List[str], **kargs):
        examples = [example.split('\t') for example in raw_documents]
        for i_sent, columns in enumerate(examples):
            if len(columns) != 2:
                print(columns)
                raise ValueError(f'Found some errors in line {i_sent}: {examples[i_sent]}')
        texts, pairs = zip(*examples)
        return texts, pairs


def get_attribute_name(path):
    if path[-4:] != '.tsv':
        raise ValueError('File must be format of tsv')
    name = os.path.basename(path)[:-4].replace('.', '_')
    return name



import json
import os
import re
from dataclasses import dataclass
from glob import glob
from tqdm import tqdm
from typing import List

from .korpora import KorpusData
from .korpus_modu_news import ModuKorpus
from .utils import default_korpora_path


class ModuMessengerKorpus(ModuKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__()
        if root_dir is None:
            root_dir = os.path.join(default_korpora_path, 'NIKL_MESSENGER')
        alternative_root_dir = os.path.join(root_dir, 'NIKL_MESSENGER')
        if os.path.exists(alternative_root_dir):
            root_dir = alternative_root_dir
        paths = find_corpus_paths(root_dir)
        self.train = KorpusData('모두의_메신저_말뭉치(conversation).train', load_modu_messenger(paths))


@dataclass
class Utterance:
    document_id: str
    form: List[str]
    original_form: List[str]
    speaker_id: List[int]
    time: List[str]

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return f'Conversation(id={self.document_id}, len={len(self.form)}, attributes=(form(str), original_form(str), speaker_id(str), time(str)))'


def document_to_utterance(document):
    document_id = document['id']
    utterance = document['utterance']
    columns = [(u['form'], u['original_form'], u['speaker_id'], u['time']) for u in utterance]
    form, original_form, speaker_id, time = zip(*columns)
    return Utterance(document_id, form, original_form, speaker_id, time)


def find_corpus_paths(root_dir_or_paths):
    prefix_pattern = re.compile('M[DM]RW')
    def match(path):
        prefix = path.split(os.path.sep)[-1][:4]
        return prefix_pattern.match(prefix)

    # directory + wildcard
    if isinstance(root_dir_or_paths, str):
        paths = sorted(glob(f'{root_dir_or_paths}/*.json') + glob(root_dir_or_paths))
    else:
        paths = root_dir_or_paths

    paths = [path for path in paths if match(path)]
    if not paths:
        raise ValueError('Not found corpus files. Check `root_dir_or_paths`')
    return paths


def load_modu_messenger(paths):
    utterances = []
    for i_path, path in enumerate(tqdm(paths, desc='Loading ModuMessenger', total=len(paths))):
        with open(path, encoding='utf-8') as f:
            data = json.load(f)
        documents = data['document']
        utterances += [document_to_utterance(doc) for doc in documents]
    return utterances



import os
from typing import List

from .korpora import Korpus, SentencePairKorpusData
from .utils import fetch, default_korpora_path, load_parallel_text


KOREAN_PARALLEL_KOEN_NEWS_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/jungyeul/korean-parallel-corpora/master/korean-english-news-v1/korean-english-park.train.tar.gz',
        'destination': 'korean_parallel/korean-english-park.train.tar.gz',
        'method': 'download & untar'
    },
    {
        'url': 'https://raw.githubusercontent.com/jungyeul/korean-parallel-corpora/master/korean-english-news-v1/korean-english-park.dev.tar.gz',
        'destination': 'korean_parallel/korean-english-park.dev.tar.gz',
        'method': 'download & untar'
    },
    {
        'url': 'https://raw.githubusercontent.com/jungyeul/korean-parallel-corpora/master/korean-english-news-v1/korean-english-park.test.tar.gz',
        'destination': 'korean_parallel/korean-english-park.test.tar.gz',
        'method': 'download & untar'
    },
]


description = """    Author : KakaoBrain
    Repository : https://github.com/jungyeul/korean-parallel-corpora
    References :
        - Jungyeul Park, Jeen-Pyo Hong and Jeong-Won Cha (2016) Korean Language Resources for Everyone.
          In Proceedings of the 30th Pacific Asia Conference on Language, Information and Computation
          (PACLIC 30). October 28 - 30, 2016. Seoul, Korea. 
          (https://www.aclweb.org/anthology/Y16-2002/)"""

license = """    Creative Commons Attribution Noncommercial No-Derivative-Works 3.0
    Details in https://creativecommons.org/licenses/by-nc-nd/3.0/"""


class KoreanParallelKOENNewsKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path

        train_info, dev_info, test_info = KOREAN_PARALLEL_KOEN_NEWS_FETCH_INFORMATION
        self.train = self.fetch_and_load('train', root_dir, train_info, force_download)
        self.dev = self.fetch_and_load('dev', root_dir, dev_info, force_download)
        self.test = self.fetch_and_load('test', root_dir, test_info, force_download)

    def fetch_and_load(self, mode, root_dir, fetch_info, force_download):
        dataname = f'koennews.{mode}'
        source_path = f'{root_dir}/korean_parallel/korean-english-park.{mode}.ko'
        target_path = f'{root_dir}/korean_parallel/korean-english-park.{mode}.en'
        if (force_download) or (not os.path.exists(source_path)) or (not os.path.exists(target_path)):
            local_path = os.path.join(os.path.abspath(root_dir), fetch_info['destination'])
            fetch(fetch_info['url'], local_path, 'korean_parallel', force_download, fetch_info['method'])
        sources, targets = load_parallel_text(source_path, target_path)
        return SentencePairKorpusData(dataname, sources, targets)


def fetch_korean_parallel_koen_news(root_dir, force_download):
    for info in KOREAN_PARALLEL_KOEN_NEWS_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'korean_parallel', force_download, info['method'])



import numpy as np
import os
from tqdm import tqdm

from .loader import Korpora
from .utils import default_korpora_path


def create_lmdata(args):
    corpus_names = check_corpus(args.corpus)
    os.makedirs(os.path.abspath(args.output_dir), exist_ok=True)

    sampling_ratio = args.sampling_ratio
    if sampling_ratio is not None:
        sampling_ratio = float(sampling_ratio)
        if not (0 < sampling_ratio < 1):
            raise ValueError('`sampling_ratio` must be None or (0, 1) float')
    n_first_samples = args.n_first_samples
    np.random.seed(args.seed)
    selector = Selector(sampling_ratio, args.min_length, args.max_length)

    root_dir = args.root_dir
    if root_dir is None:
        root_dir = default_korpora_path
    force_download = args.force_download
    multilingual = args.multilingual

    status = [['', name, ' - ', ''] for name in corpus_names]

    for i_corpus, name in enumerate(corpus_names):
        if not args.save_each and i_corpus > 0:
            mode = 'a'
        else:
            mode = 'w'

        filename = f'{name}.train' if args.save_each else 'all.train'
        lmdata_path = f'{args.output_dir}/{filename}'

        sent_iterator = tqdm(
            ITERATE_TEXTS[name](root_dir, force_download, multilingual),
            desc=f'Create train data from {name}'
        )
        print_status(status)

        n_sampled = 0
        with open(lmdata_path, mode, encoding='utf-8') as f:
            for i_sent, sent in enumerate(sent_iterator):
                if not selector.use(sent):
                    continue
                f.write(f'{sent}\n')
                n_sampled += 1
                if (n_first_samples is not None) and (n_first_samples <= n_sampled):
                    break

        status[i_corpus][0] = ' x '
        status[i_corpus][2] = n_sampled
        status[i_corpus][3] = filename
    print_status(status)


class Selector:
    def __init__(self, sampling_ratio, min_length, max_length):
        if isinstance(min_length, int) and min_length < 0:
            min_length = None
        if isinstance(max_length, int) and max_length < 0:
            max_length = None
        self.sampling_ratio = sampling_ratio
        self.min_length = min_length
        self.max_length = max_length

    def use(self, text):
        length = len(text)
        if (self.min_length is not None) and (length < self.min_length):
            return False
        if (self.max_length is not None) and (length > self.max_length):
            return False
        if self.sampling_ratio is None:
            return True
        return np.random.rand() < self.sampling_ratio


def check_corpus(corpus_names):
    if (corpus_names == 'all') or (corpus_names[0] == 'all'):
        corpus_names = list(ITERATE_TEXTS)
    if isinstance(corpus_names, str):
        corpus_names = [corpus_names]
    available = []
    for name in corpus_names:
        if name not in ITERATE_TEXTS:
            print(f'Not provide {name} corpus. Check the `corpus` argument')
            continue
        available.append(name)
    if not available:
        raise ValueError('Not found any proper corpus name. Check the `corpus` argument')
    return available


def print_status(status):
    max_len = max(max(len(row[3]) for row in status), 9)
    form = '| {:4} | {:25} | {:10} | {} |'
    print('\n\n' + form.format('Done', 'Corpus name', 'Num sents', 'File name' + ' ' * (max_len - 9)))
    print(form.format('-' * 4, '-' * 25, '-' * 10, '-' * max_len))
    for finish, name, num_sent, filename in status:
        if not filename:
            filename = ' ' * max_len
        else:
            filename += ' ' * (max_len -len(filename))
        print(form.format(finish, name, num_sent, filename))


def iterate_kcbert(root_dir, force_download, multilingual=False):
    Korpora.fetch('kcbert', root_dir, force_download)
    with open(f'{root_dir}/kcbert/20190101_20200611_v2.txt', encoding='utf-8') as f:
        for line in f:
            line = line.strip()
            if not line:
                continue
            yield line


def iterate_korean_chatbot_data(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('korean_chatbot_data', root_dir, force_download)
    for sents in [corpus.train.texts, corpus.train.pairs]:
        for sent in sents:
            if not sent:
                continue
            yield sent


def iterate_korean_hate_speech(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('korean_hate_speech', root_dir, force_download)
    for sents in [corpus.train.texts, corpus.dev.texts, corpus.unlabeled.texts]:
        for sent in sents:
            yield sent


def iterate_korean_parallel_koen_news(root_dir, force_download, multilingual):
    corpus = Korpora.load('korean_parallel_koen_news', root_dir, force_download)
    data = [corpus.train.texts, corpus.dev.texts, corpus.test.texts]
    if multilingual:
        data += [corpus.train.pairs, corpus.dev.pairs, corpus.test.pairs]
    for sents in data:
        for sent in sents:
            yield sent


def iterate_korean_petitions(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('korean_petitions', root_dir, force_download)
    for example in corpus.train:
        yield example.title
        yield example.text


def iterate_kornli(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('kornli', root_dir, force_download)
    for data in [corpus.multinli_train, corpus.snli_train, corpus.xnli_dev, corpus.xnli_test]:
        for sent in data.texts:
            yield sent
        for sent in data.pairs:
            yield sent


def iterate_korsts(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('korsts', root_dir, force_download)
    for data in [corpus.train, corpus.dev, corpus.test]:
        for sent in data.texts:
            yield sent
        for sent in data.pairs:
            yield sent


def iterate_kowikitext(root_dir, force_download, multilingual=False):
    Korpora.fetch('kowikitext', root_dir, force_download)
    paths = [
        f'{root_dir}/kowiki/kowikitext_20200920.train',
        f'{root_dir}/kowiki/kowikitext_20200920.dev',
        f'{root_dir}/kowiki/kowikitext_20200920.test'
    ]
    for path in paths:
        with open(path, encoding='utf-8') as f:
            for line in f:
                line = line.strip()
                if not line or (line[0] == '=' and line[-1] == '='):
                    continue
                yield line


def iterate_namuwikitext(root_dir, force_download, multilingual=False):
    Korpora.fetch('namuwikitext', root_dir, force_download)
    paths = [
        f'{root_dir}/namiwiki/namuwikitext_20200302.train',
        f'{root_dir}/namiwiki/namuwikitext_20200302.dev',
        f'{root_dir}/namiwiki/namuwikitext_20200302.test'
    ]
    for path in paths:
        with open(path, encoding='utf-8') as f:
            for line in f:
                line = line.strip()
                if not line or (line[0] == '=' and line[-1] == '='):
                    continue
                yield line


def iterate_naver_changwon_ner(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('naver_changwon_ner', root_dir, force_download)
    for sent in corpus.train.texts:
        yield sent


def iterate_nsmc(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('nsmc', root_dir, force_download)
    for sents in [corpus.train.texts, corpus.test.texts]:
        for sent in sents:
            yield sent


def iterate_question_pair(root_dir, force_download, multilingual=False):
    corpus = Korpora.load('question_pair', root_dir, force_download)
    for sents in [corpus.train.texts, corpus.train.pairs]:
        for sent in sents:
            yield sent


ITERATE_TEXTS = {
    'kcbert': iterate_kcbert,
    'korean_chatbot_data': iterate_korean_chatbot_data,
    'korean_hate_speech': iterate_korean_hate_speech,
    'korean_parallel_koen_news': iterate_korean_parallel_koen_news,
    'korean_petitions': iterate_korean_petitions,
    'kornli': iterate_kornli,
    'korsts': iterate_korsts,
    'kowikitext': iterate_kowikitext,
    'namuwikitext': iterate_namuwikitext,
    'naver_changwon_ner': iterate_naver_changwon_ner,
    'nsmc': iterate_nsmc,
    'question_pair': iterate_question_pair
}


import os
from typing import List

from .korpora import Korpus, LabeledSentencePairKorpusData
from .utils import fetch, default_korpora_path, load_text


KORNLI_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorNLI/multinli.train.ko.tsv',
        'destination': 'kornli/multinli.train.ko.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorNLI/snli_1.0_train.ko.tsv',
        'destination': 'kornli/snli_1.0_train.ko.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorNLI/xnli.dev.ko.tsv',
        'destination': 'kornli/xnli.dev.ko.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kakaobrain/KorNLUDatasets/master/KorNLI/xnli.test.ko.tsv',
        'destination': 'kornli/xnli.test.ko.tsv',
        'method': 'download'
    }
]


description = """    Author : KakaoBrain
    Repository : https://github.com/kakaobrain/KorNLUDatasets
    References :
        - Ham, J., Choe, Y. J., Park, K., Choi, I., & Soh, H. (2020). KorNLI and KorSTS: New Benchmark
           Datasets for Korean Natural Language Understanding. arXiv preprint arXiv:2004.03289.
           (https://arxiv.org/abs/2004.03289)

    This is the dataset repository for our paper
    "KorNLI and KorSTS: New Benchmark Datasets for Korean Natural Language Understanding."
    (https://arxiv.org/abs/2004.03289)
    We introduce KorNLI and KorSTS, which are NLI and STS datasets in Korean."""

license = """    Creative Commons Attribution-ShareAlike license (CC BY-SA 4.0)
    Details in https://creativecommons.org/licenses/by-sa/4.0/"""


class KorNLIKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_kornli(root_dir, force_download)

        for info in KORNLI_FETCH_INFORMATION:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
            if 'multinli.train' in info['destination']:
                self.multinli_train = LabeledSentencePairKorpusData(
                    'KorNLI.multinli_train',
                    *self.cleaning(load_text(local_path, num_heads=1))
                )
            elif 'snli_1.0_train' in info['destination']:
                self.snli_train = LabeledSentencePairKorpusData(
                    'KorNLI.snli_1.0_train',
                    *self.cleaning(load_text(local_path, num_heads=1))
                )
            elif 'xnli.dev' in info['destination']:
                self.xnli_dev = LabeledSentencePairKorpusData(
                    'KorNLI.xnli_dev',
                    *self.cleaning(load_text(local_path, num_heads=1))
                )
            else:
                self.xnli_test = LabeledSentencePairKorpusData(
                    'KorNLI.xnli_test',
                    *self.cleaning(load_text(local_path, num_heads=1))
                )

    def cleaning(self, raw_lines: List[str]):
        separated_lines = [line.split('\t') for line in raw_lines]
        for i_sent, separated_line in enumerate(separated_lines):
            if len(separated_line) != 3:
                raise ValueError(f'Found some errors in line {i_sent}: {separated_line}')
        texts, pairs, labels = zip(*separated_lines)
        return texts, pairs, labels

    def get_all_texts(self):
        return (self.multinli_train.get_all_texts() +
                self.snli_train.get_all_texts() +
                self.xnli_dev.get_all_texts() +
                self.xnli_test.get_all_texts())

    def get_all_pairs(self):
        return (self.multinli_train.get_all_pairs() +
                self.snli_train.get_all_pairs() +
                self.xnli_dev.get_all_pairs() +
                self.xnli_test.get_all_pairs())

    def get_all_labels(self):
        return (self.multinli_train.get_all_labels() +
                self.snli_train.get_all_labels() +
                self.xnli_dev.get_all_labels() +
                self.xnli_test.get_all_labels())


def fetch_kornli(root_dir, force_download):
    for info in KORNLI_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'kornli', force_download)



import os
import re
import urllib

from .korpora import Korpus, SentencePairKorpusData
from .utils import fetch, default_korpora_path


OPEN_SUBTITLES_FETCH_INFORMATION = [
    {
        'url': 'http://opus.nlpl.eu/download.php?f=OpenSubtitles/v2018/tmx/en-ko.tmx.gz',
        'destination': 'open_subtitles/en-ko.tmx.gz',
        'method': 'download & ungzip'
    }
]


description = """    Author : TRAC (https://trac.edgewall.org/)
    Repository : http://opus.nlpl.eu/OpenSubtitles-v2018.php
    References :
        - P. Lison and J. Tiedemann, 2016, OpenSubtitles2016: Extracting Large Parallel Corpora
          from Movie and TV Subtitles. In Proceedings of the 10th International Conference on
          Language Resources and Evaluation (LREC 2016)

    This is a new collection of translated movie subtitles from http://www.opensubtitles.org/.

    [[ IMPORTANT ]]
    If you use the OpenSubtitle corpus: Please, add a link to http://www.opensubtitles.org/
    to your website and to your reports and publications produced with the data!
    I promised this when I got the data from the providers of that website!

    This is a slightly cleaner version of the subtitle collection using improved sentence alignment
    and better language checking.

    62 languages, 1,782 bitexts
    total number of files: 3,735,070
    total number of tokens: 22.10G
    total number of sentence fragments: 3.35G

    [[ NOTICE ]]
    In original data, the source language is `en` and target language is `ko`. However in Korpora,
    we change the language pair so that source language is `ko` and target language is `en`."""

license = """    Open Data. Details in https://opendefinition.org/od/2.1/en/"""


class OpenSubtitleKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_open_subtitles(root_dir, force_download)

        sources, targets = [], []
        for info in OPEN_SUBTITLES_FETCH_INFORMATION:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])[:-3]
            sources_, targets_ = parse_xtm(local_path)
            sources += sources_
            targets += targets_
        self.train = SentencePairKorpusData('OpenSubtitle.train', targets, sources)

    def get_all_pairs(self):
        return self.train.get_all_pairs()


def parse_xtm(path):
    pattern = re.compile('<seg>[\S ]+</seg>')
    def parse_segment(line):
        seg = pattern.findall(line)[0]
        return seg[5:-6]

    sources = []
    targets = []
    mode = 0
    source, target = None, None
    with open(path, encoding='utf-8') as f:
        for line in f:
            line = line.strip()
            if line[:4] == '<tu>':
                mode += 1
                continue
            elif line[:5] == '</tu>':
                mode = 0
                if source is not None and target is not None:
                    sources.append(source)
                    targets.append(target)
                source, target = None, None
                continue
            try:
                if mode == 1:
                    source = parse_segment(line)
                    mode += 1
                    continue
                if mode == 2:
                    target = parse_segment(line)
                    continue
            except:
                mode = 0
    return sources, targets


def fetch_open_subtitles(root_dir, force_download):
    for info in OPEN_SUBTITLES_FETCH_INFORMATION:
        try:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
            fetch(info['url'], local_path, 'open_subtitles', force_download, info['method'])
        except urllib.error.HTTPError as exception:
            print('[Korpora] [open_subtitles] Failed to download. Re-try again')
            print(f'[Korpora] [open_subtitles] error messgae: {str(exception)}')
            continue



import json
import os
import re
from dataclasses import dataclass
from glob import glob
from tqdm import tqdm
from typing import List

from .korpora import Korpus, KorpusData
from .utils import default_korpora_path

description = """    모두의 말뭉치는 문화체육관광부 산하 국립국어원에서 제공하는 말뭉치로
    총 13 개의 말뭉치로 이뤄져 있습니다.

    해당 말뭉치를 이용하기 위해서는 국립국어원 홈페이지에 가셔서 "회원가입 > 말뭉치 신청 > 승인"의
    과정을 거치셔야 합니다.

    https://corpus.korean.go.kr/#none

    모두의 말뭉치는 승인 후 다운로드 가능 기간 및 횟수 (3회) 에 제한이 있습니다.

    로그인 기능 및 Korpora 패키지에서의 다운로드 기능을 제공하려 하였지만,
    국립국어원에서 위의 이유로 이에 대한 기능은 제공이 불가함을 확인하였습니다.

    Korpora==0.2.0 에서는 "개별 말뭉치 신청 > 승인"이 완료되었다고 가정,
    로컬에 다운로드 된 말뭉치를 손쉽게 로딩하는 기능만 제공합니다

    (Korpora 개발진 lovit@github, ratsgo@github)"""

license = """    모두의 말뭉치의 모든 저작권은 `문화체육관광부 국립국어원
    (National Institute of Korean Language)` 에 귀속됩니다.

    소유권을 포함한 전문은 다음의 주소에서 확인하실 수 있습니다.

    https://corpus.korean.go.kr/boards/termsInfo.do

    제13조 (소유권)
    ① 누리집이 제공하는 서비스, 그에 필요한 소프트웨어, 이미지, 마크, 로고, 디자인, 서비스 명칭,
      정보 및 상표 등과 관련된 지식 재산권 및 기타 권리는 국어원에게 있습니다.
    ② 이용자는 누리집에서 명시적으로 승인한 경우를 제외하고는 제1항의 소정의 각 재산에 대한 전부 또는
      일부의 복사, 복제, 전송, 송신, 수정, 대여, 대출, 판매, 배포, 제작, 이용 허락, 양도, 재라이선스,
      담보권 설정 행위, 상업적 이용 행위를 할 수 없으며, 제3자로 하여금 이와 같은 행위를 하도록 허락할 수 없습니다."""


class ModuKorpus(Korpus):
    def __init__(self, force_download=False):
        if force_download:
            fetch_modu()
        super().__init__(description, license)


class ModuNewsKorpus(ModuKorpus):
    def __init__(self, root_dir=None, force_download=False, load_light=True):
        super().__init__(force_download)
        if root_dir is None:
            root_dir = os.path.join(default_korpora_path, 'NIKL_NEWSPAPER')
        alternative_root_dir = os.path.join(root_dir, 'NIKL_NEWSPAPER')
        if os.path.exists(alternative_root_dir):
            root_dir = alternative_root_dir
        paths = find_corpus_paths(root_dir)
        if load_light:
            self.train = ModuNewsDataLight('모두의_뉴스_말뭉치(light).train', load_modu_news(paths, load_light))
        else:
            self.train = ModuNewsData('모두의_뉴스_말뭉치.train', load_modu_news(paths, load_light))
        self.row_to_documentid = [news.document_id for news in self.train]
        self.documentid_to_row = {document_id: idx for idx, document_id in enumerate(self.row_to_documentid)}


class ModuNewsData(KorpusData):
    def __init__(self, name, news):
        super().__init__(name, news)
        self.document_ids = [doc.document_id for doc in news]
        self.titles = [doc.title for doc in news]
        self.authors = [doc.author for doc in news]
        self.publishers = [doc.publisher for doc in news]
        self.dates = [doc.date for doc in news]
        self.topics = [doc.topic for doc in news]
        self.original_topics = [doc.original_topic for doc in news]
        self.texts = [doc.paragraph for doc in news]

    def __getitem__(self, index):
        news = ModuNews(
            self.document_ids[index],
            self.titles[index],
            self.authors[index],
            self.publishers[index],
            self.dates[index],
            self.topics[index],
            self.original_topics[index],
            self.texts[index].split('\n'))
        return news


class ModuNewsDataLight(KorpusData):
    def __init__(self, name, news):
        super().__init__(name, news)
        self.texts = [doc.paragraph for doc in news]
        self.titles = [doc.title for doc in news]
        self.document_ids = [doc.document_id for doc in news]

    def __getitem__(self, index):
        news = ModuNewsLight(
            self.document_ids[index],
            self.titles[index],
            self.texts[index])
        return news


@dataclass
class ModuNews:
    document_id: str
    title: str
    author: str
    publisher: str
    date: str
    topic: str
    original_topic: str
    paragraph: List[str]


@dataclass
class ModuNewsLight:
    document_id: str
    title: str
    paragraph: str


def document_to_a_news(document):
    document_id = document['id']
    meta = document['metadata']
    title = meta['title']
    author = meta['author']
    publisher = meta['publisher']
    date = meta['date']
    topic = meta['topic']
    original_topic = meta['original_topic']
    paragraph = '\n'.join([p['form'] for p in document['paragraph']])
    return ModuNews(document_id, title, author, publisher, date, topic, original_topic, paragraph)


def document_to_a_news_light(document):
    document_id = document['id']
    meta = document['metadata']
    title = meta['title']
    paragraph = '\n'.join([p['form'] for p in document['paragraph']])
    return ModuNewsLight(document_id, title, paragraph)


def find_corpus_paths(root_dir_or_paths):
    prefix_pattern = re.compile('N[WLPIZ]RW')
    def match(path):
        prefix = path.split(os.path.sep)[-1][:4]
        return prefix_pattern.match(prefix)

    # directory + wildcard
    if isinstance(root_dir_or_paths, str):
        paths = sorted(glob(f'{root_dir_or_paths}/*.json') + glob(root_dir_or_paths))
    else:
        paths = root_dir_or_paths

    paths = [path for path in paths if match(path)]
    if not paths:
        raise ValueError('Not found corpus files. Check `root_dir_or_paths`')
    return paths


def load_modu_news(paths, load_light):
    transform = document_to_a_news_light if load_light else document_to_a_news
    news = []
    for i_path, path in enumerate(tqdm(paths, desc='Loading ModuNews', total=len(paths))):
        with open(path, encoding='utf-8') as f:
            data = json.load(f)
        documents = data['document']
        news += [transform(document) for document in documents]
    return news


def fetch_modu(root_dir=None, force_download=False):
    raise NotImplementedError(
        "국립국어원에서 API 기능을 제공해 줄 수 없음을 확인하였습니다."
        "\n이에 따라 모두의 말뭉치는 fetch 기능을 제공하지 않습니다"
    )



from dataclasses import dataclass
from typing import List, Union


@dataclass
class KorpusData:
    name: str
    texts: List[str]

    def __len__(self):
        return len(self.texts)

    def __iter__(self):
        for i in range(len(self)):
            yield self[i]

    def __getitem__(self, index):
        return self.texts[index]

    def get_all_texts(self):
        return self.texts

    def __str__(self):
        attributes = ""
        for var_name, var in self.__dict__.items():
            if var_name not in {'name', 'description', 'self'}:
                attributes += f'  - {self.name}.{var_name} : list[{var[0].__class__.__name__}]\n'
        s = f"""{self.name}: size={len(self.texts)}\n{attributes}"""
        return s

    def __repr__(self):
        return self.__str__()


@dataclass
class LabeledSentence:
    text: str
    label: Union[int, float, str, bool]


class LabeledSentenceKorpusData(KorpusData):
    labels: List[Union[str, int]]

    def __init__(self, name, texts, labels):
        if not (len(texts) == len(labels)):
            raise ValueError('All two arguments must be same length')
        super().__init__(name, texts)
        self.labels = labels

    def __getitem__(self, index):
        return LabeledSentence(self.texts[index], self.labels[index])

    def get_all_labels(self):
        return self.labels


@dataclass
class SentencePair:
    text: str
    pair: str


class SentencePairKorpusData(KorpusData):
    pairs: List[str]

    def __init__(self, name, texts, pairs):
        if not (len(texts) == len(pairs)):
            raise ValueError('All two arguments must be same length')
        super().__init__(name, texts)
        self.pairs = pairs

    def __getitem__(self, index):
        return SentencePair(self.texts[index], self.pairs[index])

    def get_all_pairs(self):
        return self.pairs


@dataclass
class LabeledSentencePair:
    text: str
    pair: str
    label: Union[str, int, float, bool]


class LabeledSentencePairKorpusData(KorpusData):
    pairs: List[str]
    labels: List

    def __init__(self, name, texts, pairs, labels):
        if not (len(texts) == len(pairs) == len(labels)):
            raise ValueError('All three arguments must be same length')
        super().__init__(name, texts)
        self.pairs = pairs
        self.labels = labels

    def __getitem__(self, index):
        return LabeledSentencePair(self.texts[index], self.pairs[index], self.labels[index])

    def get_all_pairs(self):
        return self.pairs

    def get_all_labels(self):
        return self.labels


@dataclass
class WordTag:
    text: str
    words: List[str]
    tags: List[str]


class WordTagKorpusData(KorpusData):
    words: List[List[str]]
    tags: List[List[str]]

    def __init__(self, name, texts, words, tags):
        if not (len(texts) == len(words) == len(tags)):
            raise ValueError('All three arguments must be same length')
        super().__init__(name, texts)
        self.words = words
        self.tags = tags

    def __getitem__(self, index):
        return WordTag(self.texts[index], self.words[index], self.tags[index])

    def get_all_words(self):
        return self.words

    def get_all_tags(self):
        return self.tags


class Korpus:
    description: str
    license: str

    def __init__(self, description, license):
        self.description = description
        self.license = license
        print(f"""
    Korpora 는 다른 분들이 연구 목적으로 공유해주신 말뭉치들을
    손쉽게 다운로드, 사용할 수 있는 기능만을 제공합니다.

    말뭉치들을 공유해 주신 분들에게 감사드리며, 각 말뭉치 별 설명과 라이센스를 공유 드립니다.
    해당 말뭉치에 대해 자세히 알고 싶으신 분은 아래의 description 을 참고,
    해당 말뭉치를 연구/상용의 목적으로 이용하실 때에는 아래의 라이센스를 참고해 주시기 바랍니다.

    # Description
{description}

    # License
{license}\n""")

    def __str__(self):
        classname = self.__class__.__name__
        s = f"{classname}\n{self.description}\n\nAttributes\n----------\n"
        for var_name, var in self.__dict__.items():
            if isinstance(var, KorpusData):
                s += f'{str(var)}'
        return s

    def cleaning(self, raw_documents: List[str], **kargs):
        """`raw_data` to `sentences`"""
        raise NotImplementedError('Implement this function')

    def get_all_texts(self):
        texts = []
        for name, var in sorted(self.__dict__.items()):
            if isinstance(var, KorpusData):
                texts += var.get_all_texts()
        return texts

    def save(self, root_dir):
        """save prorce` to `sentences`"""
        raise NotImplementedError('Implement this function')



from .about import __author__   # noqa: F401
from .about import __version__  # noqa: F401

from .korpus_aihub_translation import (
    AIHubTranslationKorpus,
    AIHubSpokenTranslationKorpus,
    AIHubConversationTranslationKorpus,
    AIHubNewsTranslationKorpus,
    AIHubKoreanCultureTranslationKorpus,
    AIHubDecreeTranslationKorpus,
    AIHubGovernmentWebsiteTranslationKorpus
)
from .korpus_chatbot_data import KoreanChatbotKorpus
from .korpus_custom import (
    CustomLabeledSentencePairKorpus,
    CustomLabeledSentenceKorpus,
    CustomSentencePairKorpus
)
from .korpus_kcbert import KcBERTKorpus
from .korpus_korean_hate_speech import KoreanHateSpeechKorpus
from .korpus_korean_parallel import KoreanParallelKOENNewsKorpus
from .korpus_korean_petitions import KoreanPetitionsKorpus
from .korpus_kornli import KorNLIKorpus
from .korpus_korsts import KorSTSKorpus
from .korpus_kowiki import KowikiTextKorpus
from .korpus_namuwiki import NamuwikiTextKorpus
from .korpus_naverchangwon_ner import NaverChangwonNERKorpus
from .korpus_nsmc import NSMCKorpus
from .korpus_question_pair import QuestionPairKorpus
from .korpus_open_subtitles import OpenSubtitleKorpus
from .loader import Korpora
from .korpus_modu_news import ModuNewsKorpus
from .korpus_modu_messenger import ModuMessengerKorpus
from .korpus_modu_morpheme import ModuMorphemeKorpus
from .korpus_modu_ne import ModuNEKorpus
from .korpus_modu_spoken import ModuSpokenKorpus
from .korpus_modu_web import ModuWebKorpus
from .korpus_modu_written import ModuWrittenKorpus



import os
import xlrd
from glob import glob
from tqdm import tqdm

from Korpora.korpora import Korpus, SentencePairKorpusData
from Korpora.utils import default_korpora_path


description = """    AI Hub 에서는 학습용 데이터를 제공합니다.

    데이터를 활용하기 위해서는 아래 주소의 홈페이지에서 "AI데이터" 클릭 후,
    이용하려는 데이터마다 직접 신청을 하셔야 합니다.

    https://www.aihub.or.kr/

    AI Hub 학습데이터는 신청 즉시 자동 승인됩니다.
    Korpora>=0.2.0 에서는 로컬에 다운로드 된 말뭉치를 손쉽게 로딩하는 기능만 제공합니다.

    AI Hub 에서 제공하는 번역데이터는 압축파일 또는 엑셀파일 (확장자: xlsx) 형식입니다.
    압축 해제 시 다음의 한글로 파일 이름이 기록되어 있습니다.
    파일 이름을 한글로 기록할 경우 OS 에 따라 예상치 못한 문제들이 발생할 수 있습니다.

    그러므로 각 파일의 이름을 아래처럼 영어로 변경하였다고 가정합니다.

             한글 파일 이름                         영어 파일 이름
        --------------------------------------------------------------
        1_구어체(1)_200226.xlsx         ->  1_spoken(1)_200226.xlsx
        1_구어체(2)_200226.xlsx         ->  1_spoken(2)_200226.xlsx
        2_대화체_200226.xlsx            ->  2_conversation_200226.xlsx
        3_문어체_뉴스(1)_200226.xlsx     ->  3_news(1)_200226.xlsx
        3_문어체_뉴스(2)_200226.xlsx     ->  3_news(2)_200226.xlsx
        3_문어체_뉴스(3)_200226.xlsx     ->  3_news(3)_200226.xlsx
        3_문어체_뉴스(4)_200226.xlsx     ->  3_news(4)_200226.xlsx
        4_문어체_한국문화_200226.xlsx     ->  4_korean_culture_200226.xlsx
        5_문어체_조례_200226.xlsx        ->  5_decree_200226.xlsx
        6_문어체_지자체웹사이트_200226.xlsx ->  6_government_website_200226.xlsx

    위 파일들은 `~/Korpora/AIHub_Translation/` 혹은 `path/to/AIHub_Translation/` 에
    저장되었다고 가정합니다.

    (Korpora 개발진 lovit@github, ratsgo@github)"""

license = """    AI Hub 에서 제공하는 데이터의 소유권 및 전문은 다음의 주소에서 확인할 수 있습니다.

    https://aihub.or.kr/form/iyongyaggwan

    제16조 포털의 소유권
    1. AI 허브가 제공하는 서비스, 그에 필요한 소프트웨어, 이미지, 마크, 로고, 디자인, 서비스명칭, 정보 및
       상표 등과 관련된 지식재산권 및 기타 권리는 운영기관(및 AI허브 서비스 제공과 관련하여 운영기관과 계약을
       체결한 기관)에 소유권이 있습니다.
    2. 귀하는 AI 허브에서 명시적으로 승인한 경우를 제외하고는 전항의 소정의 각 재산에 대한 전부 또는 일부의 수정,
       대여, 대출, 판매, 배포, 제작, 양도, 재라이센스, 담보권 설정 행위, 상업적 이용 행위를 할 수 없으며,
       제3자로 하여금 이와 같은 행위를 하도록 허락할 수 없습니다"""


class AIHubTranslationKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False, prefix='', name='AIHub_translation'):
        super().__init__(description, license)
        if root_dir is None:
            root_dir = os.path.join(default_korpora_path, 'AIHub_Translation', prefix)
        elif isinstance(root_dir, str) and os.path.isdir(root_dir):
            root_dir = os.path.join(root_dir, prefix)
        paths = find_corpus_paths(root_dir)
        self.train = SentencePairKorpusData(
            f'{name}.train',
            *load_aihub_translation(paths, name)
        )


class AIHubSpokenTranslationKorpus(AIHubTranslationKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(root_dir, force_download, '1_spoken*', 'AIHub_spoken_translation')


class AIHubConversationTranslationKorpus(AIHubTranslationKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(root_dir, force_download, '2_conversation*', 'AIHub_conversation_translation')


class AIHubNewsTranslationKorpus(AIHubTranslationKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(root_dir, force_download, '3_news*', 'AIHub_news_translation')


class AIHubKoreanCultureTranslationKorpus(AIHubTranslationKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(root_dir, force_download, '4_korean_culture*', 'AIHub_korean_culture_translation')


class AIHubDecreeTranslationKorpus(AIHubTranslationKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(root_dir, force_download, '5_decree*', 'AIHub_decree_translation')


class AIHubGovernmentWebsiteTranslationKorpus(AIHubTranslationKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(root_dir, force_download, '6_government_website*', 'AIHub_government_website_translation')


def find_corpus_paths(root_dir, suffix='200226.xlsx'):
    def match(path):
        return path[-11:] == suffix

    # directory + wildcard
    if isinstance(root_dir, str):
        paths = sorted(glob(f'{root_dir}/*{suffix}') + glob(root_dir))
    else:
        paths = root_dir

    paths = [path for path in paths if match(path)]
    if not paths:
        raise ValueError('Not found corpus files. Check `root_dir`')
    return paths


def load_aihub_translation(paths, name):
    sources, targets = [], []
    for i_path, path in enumerate(tqdm(paths, desc=f'Loading {name}', total=len(paths))):
        workbook = xlrd.open_workbook(path)
        sheet = workbook.sheet_by_index(0)
        header = sheet.row(0)
        if not (header[-2].value.strip() == '원문' and header[-1].value.strip() == '번역문'):
            raise ValueError(f'The second last column and last column in header must be ("원문", "번역문")')
        for i_row in range(1, sheet.nrows):
            row = sheet.row(i_row)
            sources.append(row[-2].value.strip())
            targets.append(row[-1].value.strip())
    return sources, targets


def fetch_aihub(root_dir=None, force_download=False):
    raise NotImplementedError(
        "https://www.aihub.or.kr/ 에서 개별 신청 후 직접 데이터를 받으셔야 합니다."
        "\n이에 따라 AI Hub 의 말뭉치는 fetch 기능을 제공하지 않습니다"
    )



import json
import os
import re
from dataclasses import dataclass
from glob import glob
from tqdm import tqdm
from typing import List, Tuple

from .korpora import KorpusData
from .korpus_modu_news import ModuKorpus
from .utils import default_korpora_path


class ModuMorphemeKorpus(ModuKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__()
        if root_dir is None:
            root_dir = os.path.join(default_korpora_path, 'NIKL_MP')
        alternative_root_dir = os.path.join(root_dir, 'NIKL_MP')
        if os.path.exists(alternative_root_dir):
            root_dir = alternative_root_dir
        paths = find_corpus_paths(root_dir)
        self.train = KorpusData('모두의_형태분석_말뭉치.train', load_modu_morpheme(paths))
        self.tagmap = {
            'JKS': '주격조사',
            'JKC': '보격조사',
            'JKG': '관형격조사',
            'JKO': '목적격조사',
            'JKB': '부사격조사',
            'JKV': '호격조사',
            'JKQ': '인용격조사',
            'JX': '보조사',
            'JC': '접속조사',
            'EP': '선어말어미',
            'EF': '종결어미',
            'EC': '연결어미',
            'ETN': '명사형전성어미',
            'ETM': '관형형전성어미',
            'XPN': '체언접두사',
            'XSN': '명사파생접미사',
            'XSV': '동사파생접미사',
            'XSA': '형용사파생접미사',
            'XR': '어근',
            'SF': '마침표, 물음표, 느낌표',
            'SP': '쉼표, 가운뎃점, 콜론, 빗금',
            'SS': '따옴표, 괄호표, 줄표',
            'SE': '줄임표',
            'SO': '붙임표(물결)',
            'SW': '기타 기호',
            'SL': '외국어',
            'SH': '한자',
            'SN': '숫자',
            'NA': '분석불능범주',
            'NF': '명사추정범주',
            'NV': '용언추정범주',
            'NNG': '일반명사',
            'NNP': '고유명사',
            'NNB': '의존명사',
            'NP': '대명사',
            'NR': '수사',
            'VV': '동사',
            'VA': '형용사',
            'VX': '보조용언',
            'VCP': '긍정지정사',
            'VCN': '부정지정사',
            'MMA': '성상 관형사',
            'MMD': '지시 관형사',
            'MMN': '수 관형사',
            'MAG': '일반부사',
            'MAJ': '접속부사',
            'IC': '감탄사',
            'NAP': '이름과 같은 개인정보'
        }


def find_corpus_paths(root_dir_or_paths):
    prefix_pattern = re.compile('[NS]XMP')
    def match(path):
        prefix = path.split(os.path.sep)[-1][:4]
        return prefix_pattern.match(prefix)

    # directory + wildcard
    if isinstance(root_dir_or_paths, str):
        paths = sorted(glob(f'{root_dir_or_paths}/*.json') + glob(root_dir_or_paths))
    else:
        paths = root_dir_or_paths

    paths = [path for path in paths if match(path)]
    if not paths:
        raise ValueError('Not found corpus files. Check `root_dir_or_paths`')
    return paths


@dataclass
class MorphemesExample:
    sentence_id: str
    sentence: str
    morphemes: List[str]
    tags: List[str]
    eojeol_ids: List[int]

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return f"""Morphemes(
    id={self.sentence_id},
    sentence={self.sentence},
    tags={self.morphemes},
    positions={self.tags},
    eojeol_id={self.eojeol_ids}
)"""


def document_to_examples(document):
    examples = []
    sentence = document['sentence']
    for example in sentence:
        example_id = example['id']
        try:
            form = example['form']
            columns = [(m['form'], m['label'], m['word_id']) for m in example['morpheme']]
            morphemes, tags, eojeol_ids = zip(*columns)
            eojeol_ids = tuple(idx - 1 for idx in eojeol_ids)
            examples.append(MorphemesExample(example_id, form, morphemes, tags, eojeol_ids))
        except:
            continue
    return examples


def load_modu_morpheme(paths):
    examples = []
    for path in paths:
        with open(path, encoding='utf-8') as f:
            data = json.load(f)
        documents = data['document']
        desc = f'Loading ModuMorpheme ({os.path.basename(path)})'
        documents_iterator = tqdm(documents, desc=desc, total=len(documents))
        examples += [example for doc in documents_iterator for example in document_to_examples(doc)]
    return examples



import os
from .korpora import Korpus, SentencePairKorpusData
from .utils import fetch, default_korpora_path, load_wikitext


NAMUWIKI_FETCH_INFORMATION = [
    {
        'url': 'https://github.com/lovit/namuwikitext/releases/download/v0.3/namuwikitext_20200302.train.zip',
        'destination': 'namuwikitext/namuwikitext_20200302.train.zip',
        'method': 'download & unzip'
    },
    {
        'url': 'https://github.com/lovit/namuwikitext/releases/download/v0.3/namuwikitext_20200302.test.zip',
        'destination': 'namuwikitext/namuwikitext_20200302.test.zip',
        'method': 'download & unzip'
    },
    {
        'url': 'https://github.com/lovit/namuwikitext/releases/download/v0.3/namuwikitext_20200302.dev.zip',
        'destination': 'namuwikitext/namuwikitext_20200302.dev.zip',
        'method': 'download & unzip'
    }
]

description = """    Author : Hyunjoong Kim lovit@github
    Repository : https://github.com/lovit/namuwikitext
    References :

    나무위키의 덤프 데이터를 바탕을 제작한 wikitext 형식의 텍스트 파일입니다.
    학습 및 평가를 위하여 위키페이지 별로 train (99%), dev (0.5%), test (0.5%) 로 나뉘어져있습니다.
"""

license = "    CC BY-NC-SA 2.0 KR which Namuwiki dump dataset is licensed"


class NamuwikiTextKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_namuwikitext(root_dir, force_download)

        for information in NAMUWIKI_FETCH_INFORMATION:
            destination = information['destination']
            local_path = os.path.join(os.path.abspath(root_dir), destination[:-4])

            if 'train' in destination:
                response = input(
                    'NamuwikiText.train text file is large (5.3G).\n'
                    'If you want to load text in your memory, please insert `yes`\n'
                    'If the `INPUT` is integer, it loads only first `INPUT` sentences\n').lower()
                if (len(response) == 1 and response == 'y') or (response == 'yes'):
                    texts, titles = self.load(local_path)
                    self.train = SentencePairKorpusData('NamuwikiText.train', texts, titles)
                elif response.isdigit():
                    texts, titles = self.load(local_path, num_lines=int(response))
                    self.train = SentencePairKorpusData('NamuwikiText.train', texts, titles)
                else:
                    dirname = os.path.abspath(f'{root_dir}/namiwiki')
                    self.train = f'Namuwikitext corpus is downloaded. Open local directory {dirname}'
                    print('Continue to load `dev` and `test`')
                continue

            texts, titles = self.load(local_path)
            if 'dev' in destination:
                self.dev = SentencePairKorpusData('NamuwikiText.dev', texts, titles)
            elif 'test' in destination:
                self.test = SentencePairKorpusData('NamuwikiText.test', texts, titles)
            else:
                raise ValueError(f'Check local files')

    def load(self, path, num_lines=-1):
        def split_title_text(wikitext):
            lines = wikitext.split('\n')
            title = lines[0]
            text = '\n'.join([line.strip() for line in lines[2:] if line.strip()])
            return title, text

        wikitexts = load_wikitext(path, num_lines)
        wikitexts = [split_title_text(wikitext) for wikitext in wikitexts]
        titles, texts = zip(*wikitexts)
        # swap position
        return texts, titles


def fetch_namuwikitext(root_dir, force_download):
    for information in NAMUWIKI_FETCH_INFORMATION:
        url = information['url']
        destination = information['destination']
        local_path = os.path.join(os.path.abspath(root_dir), destination)
        fetch(url, local_path, 'namuwikitext', force_download, information['method'])



import os
import csv

from .korpora import Korpus, LabeledSentencePairKorpusData
from .utils import fetch, default_korpora_path


KOREAN_CHATBOT_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/songys/Chatbot_data/master/ChatbotData%20.csv',
        'destination': 'korean_chatbot_data/ChatbotData.csv',
        'method': 'download'
    }
]

description = """    Author : songys@github
    Repository : https://github.com/songys/Chatbot_data
    References :

    Chatbot_data_for_Korean v1.0
      1. 챗봇 트레이닝용 문답 페어 11,876개
      2. 일상다반사 0, 이별(부정) 1, 사랑(긍정) 2로 레이블링
    자세한 내용은 위의 repository를 참고하세요."""

license = """    CC0 1.0 Universal (CC0 1.0) Public Domain Dedication
    Details in https://creativecommons.org/publicdomain/zero/1.0/"""


class KoreanChatbotKorpus(Korpus):
    """ Reference: https://github.com/songys/Chatbot_data

        Example
            {
                "question": "12시 땡!",
                "answer": "하루가 또 가네요.",
                "label": 0
            }
    """
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_chatbot(root_dir, force_download)

        local_path = os.path.join(os.path.abspath(root_dir), KOREAN_CHATBOT_FETCH_INFORMATION[0]['destination'])
        with open(local_path, 'r', encoding='utf-8') as f:
            questions, answers, labels = self.cleaning(csv.reader(f, delimiter=','))
        self.train = LabeledSentencePairKorpusData('KoreanChatbot.train', questions, answers, labels)

    def cleaning(self, examples):
        next(examples) # skip head
        examples = [example for example in examples]
        for i_sent, example in enumerate(examples):
            if len(example) != 3:
                raise ValueError(f'Found some errors in line {i_sent}: {example}')
        questions, answers, labels = zip(*examples)
        labels = [int(label) for label in labels]
        return questions, answers, labels

    def get_all_pairs(self):
        return self.train.get_all_pairs()

    def get_all_labels(self):
        return self.train.get_all_labels()


def fetch_chatbot(root_dir, force_download):
    for information in KOREAN_CHATBOT_FETCH_INFORMATION:
        url = information['url']
        destination = information['destination']
        local_path = os.path.join(os.path.abspath(root_dir), destination)
        fetch(url, local_path, 'korean_chatbot_data', force_download, information['method'])


import os
from typing import List

from .korpora import Korpus, LabeledSentenceKorpusData
from .utils import fetch, load_text, default_korpora_path


NSMC_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/e9t/nsmc/master/ratings_train.txt',
        'destination': 'nsmc/ratings_train.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/e9t/nsmc/master/ratings_test.txt',
        'destination': 'nsmc/ratings_test.txt',
        'method': 'download'
    },
]

description = """    Author : e9t@github
    Repository : https://github.com/e9t/nsmc
    References : www.lucypark.kr/docs/2015-pyconkr/#39

    Naver sentiment movie corpus v1.0
    This is a movie review dataset in the Korean language.
    Reviews were scraped from Naver Movies.

    The dataset construction is based on the method noted in
    [Large movie review dataset][^1] from Maas et al., 2011.

    [^1]: http://ai.stanford.edu/~amaas/data/sentiment/"""

license = """    CC0 1.0 Universal (CC0 1.0) Public Domain Dedication
    Details in https://creativecommons.org/publicdomain/zero/1.0/"""


class NSMCKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_nsmc(root_dir, force_download)

        for info in NSMC_FETCH_INFORMATION:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
            text, labels = self.cleaning(load_text(local_path, num_heads=1))
            if 'train' in info['destination']:
                self.train = LabeledSentenceKorpusData('NSMC.train', text, labels)
            else:
                self.test = LabeledSentenceKorpusData('NSMC.test', text, labels)

    def cleaning(self, raw_lines: List[str]):
        separated_lines = [line.split('\t') for line in raw_lines]
        for i_sent, separated_line in enumerate(separated_lines):
            if len(separated_line) != 3:
                raise ValueError(f'Found some errors in line {i_sent}: {separated_line}')
        _, texts, labels = zip(*separated_lines)
        labels = [int(label) for label in labels]
        return texts, labels

    def get_all_texts(self):
        return self.train.texts + self.test.texts

    def get_all_labels(self):
        return self.train.labels + self.test.labels


def fetch_nsmc(root_dir, force_download):
    for info in NSMC_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'nsmc', force_download)



import os
import json
from dataclasses import dataclass
from typing import List

from .korpora import Korpus, KorpusData
from .utils import fetch, default_korpora_path, load_text


KOREAN_PETITIONS_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2017-08',
        'destination': 'korean_petitions/petitions_2017-08',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2017-09',
        'destination': 'korean_petitions/petitions_2017-09',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2017-10',
        'destination': 'korean_petitions/petitions_2017-10',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2017-11',
        'destination': 'korean_petitions/petitions_2017-11',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2017-12',
        'destination': 'korean_petitions/petitions_2017-12',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-01',
        'destination': 'korean_petitions/petitions_2018-01',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-02',
        'destination': 'korean_petitions/petitions_2018-02',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-03',
        'destination': 'korean_petitions/petitions_2018-03',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-04',
        'destination': 'korean_petitions/petitions_2018-04',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-05',
        'destination': 'korean_petitions/petitions_2018-05',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-06',
        'destination': 'korean_petitions/petitions_2018-06',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-07',
        'destination': 'korean_petitions/petitions_2018-07',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-08',
        'destination': 'korean_petitions/petitions_2018-08',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-09',
        'destination': 'korean_petitions/petitions_2018-09',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-10',
        'destination': 'korean_petitions/petitions_2018-10',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-11',
        'destination': 'korean_petitions/petitions_2018-11',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2018-12',
        'destination': 'korean_petitions/petitions_2018-12',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2019-01',
        'destination': 'korean_petitions/petitions_2019-01',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2019-02',
        'destination': 'korean_petitions/petitions_2019-02',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/lovit/petitions_archive/archive/petitions_2019-03',
        'destination': 'korean_petitions/petitions_2019-03',
        'method': 'download'
    },
]


description = """    Author : Hyunjoong Kim lovit@github
    Repository : https://github.com/lovit/petitions_archive
    References :

    청와대 국민청원 게시판의 데이터를 월별로 수집한 것입니다.
    청원은 게시판에 글을 올린 뒤, 한달 간 청원이 진행됩니다.
    수집되는 데이터는 청원종료가 된 이후의 데이터이며, 청원 내 댓글은 수집되지 않습니다.
    단 청원의 동의 개수는 수집됩니다.
    자세한 내용은 위의 repository를 참고하세요."""

license = """    CC0 1.0 Universal (CC0 1.0) Public Domain Dedication
    Details in https://creativecommons.org/publicdomain/zero/1.0/"""


@dataclass
class KoreanPetition:
    text: str
    category: str
    num_agree: int
    begin: str
    end: str
    title: str


class KoreanPetitionsData(KorpusData):
    categories: List[str]
    num_agrees: List[int]
    begins: List[str]
    ends: List[str]
    titles: List[str]

    def __init__(self, dataname, contents, categories, begins, ends, num_agrees, titles):
        if not (len(contents) ==
                len(categories) ==
                len(begins) ==
                len(ends) ==
                len(num_agrees) ==
                len(titles)
               ):
            raise ValueError('All length of input arguments must be same.')

        super().__init__(dataname, texts=contents)
        self.categories = categories
        self.num_agrees = num_agrees
        self.begins = begins
        self.ends = ends
        self.titles = titles

    def __getitem__(self, index):
        return KoreanPetition(
            self.texts[index],
            self.categories[index],
            self.num_agrees[index],
            self.begins[index],
            self.ends[index],
            self.titles[index]
        )


class KoreanPetitionsKorpus(Korpus):
    """ Reference: https://github.com/lovit/petitions_archive

        Examples in a petition

            {
                "category": "일자리",
                "begin": "2018-05-01",
                "end": "2018-05-31",
                "content": "CONTENT TEXT',
                "num_agree": 2560,
                "petition_idx": "216521",
                "status": "청원종료",
                "title": "TITLE"
            }
    """
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_korean_petitions(root_dir, force_download)

        contents, categories, begins, ends, num_agrees, titles = [], [], [], [], [], []
        for info in KOREAN_PETITIONS_FETCH_INFORMATION:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
            con, cat, b, e, num, tit = self.cleaning(load_text(local_path))
            categories += cat
            contents += con
            begins += b
            ends += e
            num_agrees += num
            titles += tit
        self.train = KoreanPetitionsData(
            'KoreanPetitions.train', contents, categories,
            begins, ends, num_agrees, titles)

    def cleaning(self, raw_lines: List[str]):
        def parse(json_line):
            data = json.loads(json_line)
            return (data.get('content', None),
                    data.get('category', None),
                    data.get('begin', None),
                    data.get('end', None),
                    data.get('num_agree', 0),
                    data.get('title', None))

        separated_lines = [parse(line) for line in raw_lines]
        contents, categories, begins, ends, num_agrees, titles = zip(*separated_lines)
        return contents, categories, begins, ends, num_agrees, titles

    def get_all_categories(self):
        return self.train.categories

    def get_all_num_agrees(self):
        return self.train.num_agrees

    def get_all_titles(self):
        return self.train.titles


def fetch_korean_petitions(root_dir, force_download):
    for info in KOREAN_PETITIONS_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'korean_petitions', force_download)



import os
import platform

from .korpora import Korpus, KorpusData
from .utils import fetch, load_text, default_korpora_path


KCBERT_FETCH_INFORMATION = [
    {
        'url': 'https://github.com/Beomi/KcBERT/releases/download/TrainData_v1/kcbert-train.tar.gzaa',
        'destination': 'kcbert/kcbert-train.tar.gzaa',
        'method': 'download'
    },
    {
        'url': 'https://github.com/Beomi/KcBERT/releases/download/TrainData_v1/kcbert-train.tar.gzab',
        'destination': 'kcbert/kcbert-train.tar.gzab',
        'method': 'download'
    },
    {
        'url': 'https://github.com/Beomi/KcBERT/releases/download/TrainData_v1/kcbert-train.tar.gzac',
        'destination': 'kcbert/kcbert-train.tar.gzac',
        'method': 'download'
    }
]

description = """    Author : beomi@github
    Repository : https://github.com/Beomi/KcBERT/
    References :

    공개된 한국어 BERT는 대부분 한국어 위키, 뉴스 기사, 책 등 잘 정제된 데이터를 기반으로 학습한 모델입니다.

    한편, 실제로 NSMC와 같은 댓글형 데이터셋은 정제되지 않았고 구어체 특징에 신조어가 많으며,
    오탈자 등 공식적인 글쓰기에서 나타나지 않는 표현들이 빈번하게 등장합니다.

    KcBERT는 위와 같은 특성의 데이터셋에 적용하기 위해, 네이버 뉴스에서 댓글과 대댓글을 수집해,
    토크나이저와 BERT모델을 처음부터 학습한 Pretrained BERT 모델입니다.

    KcBERT는 Huggingface의 Transformers 라이브러리를 통해 간편히 불러와 사용할 수 있습니다.
    (별도의 파일 다운로드가 필요하지 않습니다.)"""

license = """    MIT License"""


class KcBERTKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_kcbert(root_dir, force_download)

        response = input(
            'KcBERT text file is large (12G).\n'
            'If you want to load text in your memory, please insert `yes`\n'
            'If the `INPUT` is integer, it loads only first `INPUT` sentences\n').lower()
        if (len(response) == 1 and response == 'y') or (response == 'yes'):
            self.train = KorpusData(
                'KcBERT.train',
                load_text(f'{root_dir}/kcbert/20190101_20200611_v2.txt')
            )
        elif response.isdigit():
            self.train = KorpusData(
                'KcBERT.train',
                load_text(f'{root_dir}/kcbert/20190101_20200611_v2.txt', num_samples=int(response))
            )
        else:
            dirname = os.path.abspath(f'{root_dir}/kcbert')
            self.train = f'KcBERT corpus is downloaded. Open local directory {dirname}'


def fetch_kcbert(root_dir, force_download):
    for info in KCBERT_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'kcbert', force_download)

    if os.path.exists(f'{root_dir}/kcbert/20190101_20200611_v2.txt'):
        return None

    if platform.system().lower() == 'windows':
        print('Korpora does not support KcBERT fetch in Windows OS.'
              f'Please open local directory {root_dir} and unzip manually tar files')
        return None

    cwd = os.getcwd()
    print('Unzip tar. It needs a few minutes ... ', end='')
    os.chdir(f'{root_dir}/kcbert/')
    os.system("cat kcbert-train.tar.gz* | tar -zxvpf -")
    print('done')
    os.chdir(cwd)



from .loader import Korpora, KORPUS_DESCRIPTION


def fetch(args):
    corpus_names = args.corpus
    if corpus_names == 'all':
        Korpora.fetch(corpus_names, args.root, args.force_download)
        return

    if isinstance(corpus_names, str):
        corpus_names = [corpus_names]
    for name in corpus_names:
        if name not in KORPUS_DESCRIPTION:
            print(f'Korpora does not provide `{name}` corpus')
            continue
        Korpora.fetch(name, args.root, args.force_download)



import argparse
import os

from .about import __version__
from .loader import KORPUS_DESCRIPTION
from .task_fetch import fetch
from .task_lmdata import create_lmdata


def listup(args):
    for name, description in KORPUS_DESCRIPTION.items():
        print(f'[Corpus] {name} : {description}')


def show_arguments(args):
    print('## Arguments of Korpora CLI ##')
    for name, var in sorted(vars(args).items()):
        if callable(var):
            print(f'  - {name} : {var.__name__}')
        else:
            print(f'  - {name} : {var}')


def show_version(args):
    print(f'Korpora=={__version__}')
    print('Execute `korpora --help` to see what Korpora provides')


def main():
    parser = argparse.ArgumentParser(description='Korpora Command Line Interface')
    parser.set_defaults(func=show_version)
    subparsers = parser.add_subparsers(help='Korpora')

    # fetch
    parser_fetch = subparsers.add_parser('fetch', help='Fetch `corpus` to `root`')
    parser_fetch.add_argument('--corpus', type=str, default='all', nargs='+', help='corpus name')
    parser_fetch.add_argument('--root', type=str, default=None, help='path/to/Korpora/')
    parser_fetch.add_argument('--force_download', dest='force_download', action='store_true')
    parser_fetch.set_defaults(func=fetch)

    # list
    parser_list = subparsers.add_parser('list', help='Tokenize `input` and save the result to `output`')
    parser_list.set_defaults(func=listup)

    # create language model train data
    parser_lmdata = subparsers.add_parser('lmdata', help='Create language model train data')
    parser_lmdata.add_argument('--corpus', type=str, required=True, nargs='+', help='corpus names')
    parser_lmdata.add_argument('--root_dir', type=str, default=None, help='path/to/Korpora')
    parser_lmdata.add_argument('--output_dir', type=str, required=True, help='output file path')
    parser_lmdata.add_argument('--sampling_ratio', type=float, default=None, help='Sampling ratio')
    parser_lmdata.add_argument('--n_first_samples', type=int, default=None, help='Number of first samples')
    parser_lmdata.add_argument('--min_length', type=int, default=None, help='Mininum length of text')
    parser_lmdata.add_argument('--max_length', type=int, default=None, help='Maximum length of text')
    parser_lmdata.add_argument('--seed', type=int, default=None, help='Random seed')
    parser_lmdata.add_argument('--force_download', dest='force_download', action='store_true')
    parser_lmdata.add_argument('--multilingual', dest='multilingual', action='store_true', help='If True, make include train data foreign language text')
    parser_lmdata.add_argument('--save_each', dest='save_each', action='store_true', help='store each corpus as a file')
    parser_lmdata.set_defaults(func=create_lmdata)

    # Do task
    args = parser.parse_args()
    show_arguments(args)
    task_function = args.func
    task_function(args)


if __name__ == '__main__':
    main()



from .korpus_aihub_translation import (
    AIHubTranslationKorpus,
    AIHubSpokenTranslationKorpus,
    AIHubConversationTranslationKorpus,
    AIHubNewsTranslationKorpus,
    AIHubKoreanCultureTranslationKorpus,
    AIHubDecreeTranslationKorpus,
    AIHubGovernmentWebsiteTranslationKorpus,
    fetch_aihub
)
from .korpus_chatbot_data import KoreanChatbotKorpus, fetch_chatbot
from .korpus_kcbert import KcBERTKorpus, fetch_kcbert
from .korpus_korean_hate_speech import KoreanHateSpeechKorpus, fetch_korean_hate_speech
from .korpus_korean_parallel import KoreanParallelKOENNewsKorpus, fetch_korean_parallel_koen_news
from .korpus_korean_petitions import KoreanPetitionsKorpus, fetch_korean_petitions
from .korpus_kornli import KorNLIKorpus, fetch_kornli
from .korpus_korsts import KorSTSKorpus, fetch_korsts
from .korpus_kowiki import KowikiTextKorpus, fetch_kowikitext
from .korpus_namuwiki import NamuwikiTextKorpus, fetch_namuwikitext
from .korpus_naverchangwon_ner import NaverChangwonNERKorpus, fetch_naverchangwon_ner
from .korpus_nsmc import NSMCKorpus, fetch_nsmc
from .korpus_question_pair import QuestionPairKorpus, fetch_questionpair
from .korpus_open_subtitles import OpenSubtitleKorpus, fetch_open_subtitles
from .korpus_modu_news import ModuNewsKorpus, fetch_modu
from .korpus_modu_messenger import ModuMessengerKorpus
from .korpus_modu_morpheme import ModuMorphemeKorpus
from .korpus_modu_ne import ModuNEKorpus
from .korpus_modu_spoken import ModuSpokenKorpus
from .korpus_modu_web import ModuWebKorpus
from .korpus_modu_written import ModuWrittenKorpus
from .utils import default_korpora_path


class Korpora:
    """
    Examples::
        >>> from Korpora import Korpora
        >>> nsmc = Korpora.load('nsmc')
        >>> len(nsmc.train.texts)   # 150000
        >>> len(nsmc.train.labels)  # 50000
    """
    @classmethod
    def load(cls, corpus_names, root_dir=None, force_download=False):
        return_single = isinstance(corpus_names, str)
        if return_single:
            corpus_names = [corpus_names]
        corpora = [KORPUS[corpus_name](root_dir, force_download) for corpus_name in corpus_names]
        if return_single:
            return corpora[0]
        return corpora

    @classmethod
    def fetch(cls, corpus_name, root_dir=None, force_download=False):
        if corpus_name.lower() == 'all':
            corpus_name = sorted(FETCH.keys())

        if isinstance(corpus_name, str):
            corpus_name = [corpus_name]

        corpus_name = [name for name in corpus_name if (name[:5] != 'modu_' and name[:6] != 'aihub_')]
        for name in corpus_name:
            if name not in FETCH:
                raise ValueError(f'Support only f{sorted(FETCH.keys())}')

        if root_dir is None:
            root_dir = default_korpora_path

        for name in corpus_name:
            fetch_func = FETCH[name]
            fetch_func(root_dir, force_download)

    @classmethod
    def corpus_list(cls):
        return KORPUS_DESCRIPTION


KORPUS = {
    'kcbert': KcBERTKorpus,
    'korean_chatbot_data': KoreanChatbotKorpus,
    'korean_hate_speech': KoreanHateSpeechKorpus,
    'korean_parallel_koen_news': KoreanParallelKOENNewsKorpus,
    'korean_petitions': KoreanPetitionsKorpus,
    'kornli': KorNLIKorpus,
    'korsts': KorSTSKorpus,
    'kowikitext': KowikiTextKorpus,
    'namuwikitext': NamuwikiTextKorpus,
    'naver_changwon_ner': NaverChangwonNERKorpus,
    'nsmc': NSMCKorpus,
    'question_pair': QuestionPairKorpus,
    'modu_news': ModuNewsKorpus,
    'modu_messenger': ModuMessengerKorpus,
    'modu_mp': ModuMorphemeKorpus,
    'modu_ne': ModuNEKorpus,
    'modu_spoken': ModuSpokenKorpus,
    'modu_web': ModuWebKorpus,
    'modu_written': ModuWrittenKorpus,
    'open_subtitles': OpenSubtitleKorpus,
    'aihub_translation': AIHubTranslationKorpus,
    'aihub_spoken_translation': AIHubSpokenTranslationKorpus,
    'aihub_conversation_translation': AIHubConversationTranslationKorpus,
    'aihub_news_translation': AIHubNewsTranslationKorpus,
    'aihub_korean_culture_translation': AIHubKoreanCultureTranslationKorpus,
    'aihub_decree_translation': AIHubDecreeTranslationKorpus,
    'aihub_government_website_translation': AIHubGovernmentWebsiteTranslationKorpus,
}

KORPUS_DESCRIPTION = {
    'kcbert': "beomi@github 님이 만드신 KcBERT 학습데이터",
    'korean_chatbot_data': "songys@github 님이 만드신 챗봇 문답 데이터",
    'korean_hate_speech': "{inmoonlight,warnikchow,beomi}@github 님이 만드신 혐오댓글데이터",
    'korean_parallel_koen_news': "jungyeul@github 님이 만드신 병렬 말뭉치",
    'korean_petitions': "lovit@github 님이 만드신 2017.08 ~ 2019.03 청와대 청원데이터",
    'kornli': "KakaoBrain 에서 제공하는 Natural Language Inference (NLI) 데이터",
    'korsts': "KakaoBrain 에서 제공하는 Semantic Textual Similarity (STS) 데이터",
    'kowikitext': "lovit@github 님이 만드신 wikitext 형식의 한국어 위키피디아 데이터",
    'namuwikitext': "lovit@github 님이 만드신 wikitext 형식의 나무위키 데이터",
    'naver_changwon_ner': "네이버 + 창원대 NER shared task data",
    'nsmc': "e9t@github 님이 만드신 Naver sentiment movie corpus v1.0",
    'question_pair': "songys@github 님이 만드신 질문쌍(Paired Question v.2)",
    'modu_news': '국립국어원에서 만든 모두의 말뭉치: 뉴스 말뭉치',
    'modu_messenger': '국립국어원에서 만든 모두의 말뭉치: 메신저 말뭉치',
    'modu_mp': '국립국어원에서 만든 모두의 말뭉치: 형태 분석 말뭉치',
    'modu_ne': '국립국어원에서 만든 모두의 말뭉치: 개체명 분석 말뭉치',
    'modu_spoken': '국립국어원에서 만든 모두의 말뭉치: 구어 말뭉치',
    'modu_web': '국립국어원에서 만든 모두의 말뭉치: 웹 말뭉치',
    'modu_written': '국립국어원에서 만든 모두의 말뭉치: 문어 말뭉치',
    'open_subtitles': 'Open parallel corpus (OPUS) 에서 제공하는 영화 자막 번역 병렬 말뭉치',
    'aihub_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (구어 + 대화 + 뉴스 + 한국문화 + 조례 + 지자체웹사이트)",
    'aihub_spoken_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (구어)",
    'aihub_conversation_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (대화)",
    'aihub_news_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (뉴스)",
    'aihub_korean_culture_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (한국문화)",
    'aihub_decree_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (조례)",
    'aihub_government_website_translation': "AI Hub 에서 제공하는 번역용 병렬 말뭉치 (지자체웹사이트)",
}

FETCH = {
    'kcbert': fetch_kcbert,
    'korean_chatbot_data': fetch_chatbot,
    'korean_hate_speech': fetch_korean_hate_speech,
    'korean_parallel_koen_news': fetch_korean_parallel_koen_news,
    'korean_petitions': fetch_korean_petitions,
    'kornli': fetch_kornli,
    'korsts': fetch_korsts,
    'kowikitext': fetch_kowikitext,
    'namuwikitext': fetch_namuwikitext,
    'naver_changwon_ner': fetch_naverchangwon_ner,
    'nsmc': fetch_nsmc,
    'question_pair': fetch_questionpair,
    'modu_news': fetch_modu,
    'modu_messenger': fetch_modu,
    'modu_mp': fetch_modu,
    'modu_ne': fetch_modu,
    'modu_spoken': fetch_modu,
    'modu_web': fetch_modu,
    'modu_written': fetch_modu,
    'open_subtitles': fetch_open_subtitles,
    'aihub_translation': fetch_aihub,
    'aihub_spoken_translation': fetch_aihub,
    'aihub_conversation_translation': fetch_aihub,
    'aihub_news_translation': fetch_aihub,
    'aihub_korean_culture_translation': fetch_aihub,
    'aihub_decree_translation': fetch_aihub,
    'aihub_government_website_translation': fetch_aihub,
}


import os
from setuptools import setup, find_packages


with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def get_about():
    about = {}
    basedir = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(basedir, 'Korpora', 'about.py')) as f:
        exec(f.read(), about)
    return about


def requirements():
    with open(os.path.join(os.path.dirname(__file__), 'requirements.txt'), encoding='utf-8') as f:
        return f.read().splitlines()


about = get_about()
setup(
    name="Korpora",
    version=about['__version__'],
    author=about['__author__'],
    url='https://github.com/ko-nlp/Korpora',
    description=about['__description__'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=requirements(),
    keywords=[],
    packages=find_packages(),
    entry_points = {
        'console_scripts': ['korpora=Korpora.cli:main'],
    }
)



import os
from dataclasses import dataclass
from typing import List

from .korpora import Korpus, KorpusData, SentencePairKorpusData
from .utils import fetch, default_korpora_path, load_text


KOREAN_HATE_SPEECH_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/unlabeled/unlabeled_comments_1.txt',
        'destination': 'korean_hate_speech/unlabeled/unlabeled_comments_1.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/unlabeled/unlabeled_comments_2.txt',
        'destination': 'korean_hate_speech/unlabeled/unlabeled_comments_2.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/unlabeled/unlabeled_comments_3.txt',
        'destination': 'korean_hate_speech/unlabeled/unlabeled_comments_3.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/unlabeled/unlabeled_comments_4.txt',
        'destination': 'korean_hate_speech/unlabeled/unlabeled_comments_4.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/unlabeled/unlabeled_comments_5.txt',
        'destination': 'korean_hate_speech/unlabeled/unlabeled_comments_5.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/unlabeled_comments.news_title_1.txt',
        'destination': 'korean_hate_speech/news_title/unlabeled_comments.news_title_1.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/unlabeled_comments.news_title_2.txt',
        'destination': 'korean_hate_speech/news_title/unlabeled_comments.news_title_2.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/unlabeled_comments.news_title_3.txt',
        'destination': 'korean_hate_speech/news_title/unlabeled_comments.news_title_3.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/unlabeled_comments.news_title_4.txt',
        'destination': 'korean_hate_speech/news_title/unlabeled_comments.news_title_4.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/unlabeled_comments.news_title_5.txt',
        'destination': 'korean_hate_speech/news_title/unlabeled_comments.news_title_5.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/dev.news_title.txt',
        'destination': 'korean_hate_speech/news_title/dev.news_title.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/test.news_title.txt',
        'destination': 'korean_hate_speech/news_title/test.news_title.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/news_title/train.news_title.txt',
        'destination': 'korean_hate_speech/news_title/train.news_title.txt',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/labeled/dev.tsv',
        'destination': 'korean_hate_speech/labeled/dev.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/labeled/train.tsv',
        'destination': 'korean_hate_speech/labeled/train.tsv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/test.no_label.tsv',
        'destination': 'korean_hate_speech/test.no_label.tsv',
        'method': 'download'
    }
]

description = """    Authors :
        - Jihyung Moon* (inmoonlight@github)
        - Won Ik Cho* (warnikchow@github)
        - Junbum Lee (beomi@github)
        * equal contribution
    Repository : https://github.com/kocohub/korean-hate-speech
    References :
        - Moon, J., Cho, W. I., & Lee, J. (2020). BEEP! Korean Corpus of Online News
          Comments for Toxic Speech Detection. arXiv preprint arXiv:2005.12503.

    We provide the first human-annotated Korean corpus for toxic speech detection and the large unlabeled corpus.
    The data is comments from the Korean entertainment news aggregation platform."""

license = """    Creative Commons Attribution-ShareAlike 4.0 International License.
    Visit here for detail : https://creativecommons.org/licenses/by-sa/4.0/"""


@dataclass
class KoreanHateSpeechLabeledExample:
    text: str
    title: str
    gender_bias: str
    bias: str
    hate: str


class KoreanHateSpeechLabeledData(KorpusData):
    titles: List[str]
    gender_biases: List[str]
    biases: List[str]
    hates: List[str]

    def __init__(self, dataname, texts, titles, gender_biases, biases, hates):
        super().__init__(dataname, texts)
        if not (len(texts) == len(titles) == len(gender_biases) == len(biases) == len(hates)):
            raise ValueError('All five arguments must be same length')
        self.titles = titles
        self.gender_biases = gender_biases
        self.biases = biases
        self.hates = hates

    def __getitem__(self, index):
        return KoreanHateSpeechLabeledExample(
            self.texts[index],
            self.titles[index],
            self.gender_biases[index],
            self.biases[index],
            self.hates[index]
        )


class KoreanHateSpeechKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_korean_hate_speech(root_dir, force_download)

        self.train = load_train(root_dir)
        self.dev = load_dev(root_dir)
        self.unlabeled = load_unlabeled(root_dir)
        self.test = load_test(root_dir)


def load_train(root_dir):
    # head : comments, contain_gender_bias, bias, hate
    text_labels = load_text(f'{root_dir}/korean_hate_speech/labeled/train.tsv', num_heads=1)
    titles = load_text(f'{root_dir}/korean_hate_speech/news_title/train.news_title.txt')
    if len(text_labels) != len(titles):
        raise ValueError(f'Found len(train.texts) != len(train.pairs). Do `fetch(force_download=True)`')
    texts, gender_biases, biases, hates = zip(*[line.split('\t') for line in text_labels])
    return KoreanHateSpeechLabeledData('KoreanHateSpeech.train', texts, titles, gender_biases, biases, hates)


def load_dev(root_dir):
    # head : comments, contain_gender_bias, bias, hate
    text_labels = load_text(f'{root_dir}/korean_hate_speech/labeled/dev.tsv', num_heads=1)
    titles = load_text(f'{root_dir}/korean_hate_speech/news_title/dev.news_title.txt')
    if len(text_labels) != len(titles):
        raise ValueError(f'Found len(dev.texts) != len(dev.pairs). Do `fetch(force_download=True)`')
    texts, gender_biases, biases, hates = zip(*[line.split('\t') for line in text_labels])
    return KoreanHateSpeechLabeledData('KoreanHateSpeech.dev', texts, titles, gender_biases, biases, hates)


def load_unlabeled(root_dir):
    text_paths = [f'{root_dir}/korean_hate_speech/news_title/unlabeled_comments.news_title_{i}.txt' for i in range(1, 6)]
    title_paths = [f'{root_dir}/korean_hate_speech/news_title/unlabeled_comments.news_title_{i}.txt' for i in range(1, 6)]
    texts = []
    titles = []
    for text_path, title_path in zip(text_paths, title_paths):
        texts_ = load_text(text_path)
        titles_ = load_text(title_path)
        if len(texts_) != len(titles_):
            raise ValueError(f'Found some errors. not equal num of texts and titles. Do `fetch(force_download=True)`')
        texts += texts_
        titles += titles_
    return SentencePairKorpusData('KoreanHateSpeech.unlabeled', texts, titles)


def load_test(root_dir):
    text_path = f'{root_dir}/korean_hate_speech/test.no_label.tsv'
    title_path = f'{root_dir}/korean_hate_speech/news_title/test.news_title.txt'
    texts = load_text(text_path, num_heads=1)
    titles = load_text(title_path)
    if len(texts) != len(titles):
        raise ValueError(f'Found some errors. not equal num of texts and titles. Do `fetch(force_download=True)`')
    return SentencePairKorpusData('KoreanHateSpeech.test', texts, titles)


def fetch_korean_hate_speech(root_dir, force_download):
    for info in KOREAN_HATE_SPEECH_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'korean hate speech', force_download)



import os
import csv

from .korpora import Korpus, LabeledSentencePairKorpusData
from .utils import fetch, default_korpora_path


QUESTION_PAIR_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/songys/Question_pair/master/kor_pair_train.csv',
        'destination': 'question_pair/kor_pair_train.csv',
        'method': 'download'
    },
    {
        'url': 'https://raw.githubusercontent.com/songys/Question_pair/master/kor_Pair_test.csv',
        'destination': 'question_pair/kor_pair_test.csv',
        'method': 'download'
    },
]

description = """    Author : songys@github
    Repository : https://github.com/songys/Question_pair
    References :

    질문쌍(Paired Question v.2)
    짝 지어진 두 개의 질문이 같은 질문인지 다른 질문인지 핸드 레이블을 달아둔 데이터
    사랑, 이별, 또는 일상과 같은 주제로 도메인 특정적이지 않음"""

license = """    Creative Commons Attribution-ShareAlike license (CC BY-SA 4.0)
    Details in https://creativecommons.org/licenses/by-sa/4.0/"""


class QuestionPairKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_questionpair(root_dir, force_download)

        for info in QUESTION_PAIR_FETCH_INFORMATION:
            local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
            is_train = 'train' in info['destination']
            with open(local_path, 'r', encoding='utf-8') as f:
                texts, pairs, labels = self.cleaning(csv.reader(f, delimiter=','), is_train)
                dataname = 'QuestionPair.train' if is_train else 'QuestionPair.test'
                data = LabeledSentencePairKorpusData(dataname, texts, pairs, labels)
            if is_train:
                self.train = data
            else:
                self.test = data

    def cleaning(self, examples, is_train):
        next(examples)  # skip head
        examples = [example for example in examples]
        for i, example in enumerate(examples):
            if (is_train and len(example) != 6) or (not is_train and len(example) != 5):
                raise ValueError(f'Found some errors in line {i}: {example}')
        if is_train:
            # train data column names : id, qid1, qid2, question1, question2, is_duplicate
            _, _, _, texts, pairs, labels = zip(*examples)
        else:
            # test data column names : test_id, question1, question2, is_duplicate, blank
            _, texts, pairs, labels, _ = zip(*examples)
        return texts, pairs, labels

    def get_all_texts(self):
        return self.train.get_all_texts() + self.test.get_all_texts()

    def get_all_pairs(self):
        return self.train.get_all_pairs() + self.test.get_all_pairs()

    def get_all_labels(self):
        return self.train.get_all_labels() + self.test.get_all_labels()


def fetch_questionpair(root_dir, force_download):
    for info in QUESTION_PAIR_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'question_pair', force_download)



import os
import requests
import tarfile
import zipfile
from os.path import expanduser
from tqdm import tqdm
from urllib import request

import gzip
import shutil

default_korpora_path = f'{expanduser("~")}/Korpora/'
GOOGLE_DRIVE_URL = "https://docs.google.com/uc?export=download"


def check_path(path):
    return os.path.exists(path)


def check_dir(filepath):
    dirname = os.path.abspath(os.path.dirname(filepath))
    if not os.path.exists(dirname):
        os.makedirs(dirname)


def load_text(path, num_heads=0, num_samples=-1):
    lines = []
    with open(path, encoding='utf-8') as f:
        if num_heads > 0:
            for _ in range(num_heads):
                next(f)
        for i, line in enumerate(f):
            if (num_samples > 0) and (i >= num_samples):
                break
            lines.append(line.rstrip('\n'))
    return lines


def load_parallel_text(source_path, target_path, num_heads=0, num_samples=-1):
    sources = load_text(source_path, num_heads, num_samples)
    targets = load_text(target_path, num_heads, num_samples)
    if len(sources) != len(targets):
        raise ValueError('Parallel corpus must have same length two files')
    return sources, targets


def load_wikitext(path, num_lines=-1):
    """
    Wikitext format

         = Head1 =

        text ...
        text ...

         = = 2ead = =

        text ...
        text ...
    """
    if num_lines <= 0:
        with open(path, encoding='utf-8') as f:
            texts = f.read().split('\n =')
    else:
        lines = []
        with open(path, encoding='utf-8') as f:
            for i, line in enumerate(f):
                if (i >= num_lines):
                    break
                lines.append(line)
        texts = ''.join(lines).split('\n =')
    # fix missing prefix
    texts = [texts[0]] + [f' ={text}' for text in texts[1:]]
    return texts


def _reporthook(t):
    """ ``reporthook`` to use with ``urllib.request`` that prints the process of the download.

    Uses ``tqdm`` for progress bar.

    **Reference:**
    https://github.com/tqdm/tqdm

    Args:
        t (tqdm.tqdm) Progress bar.

    Example:
        >>> with tqdm(unit='B', unit_scale=True, miniters=1, desc=filename) as t:  # doctest: +SKIP
        ...   urllib.request.urlretrieve(file_url, filename=full_path, reporthook=reporthook(t))
    """
    last_b = [0]

    def inner(b=1, bsize=1, tsize=None):
        """
        Args:
            b (int, optional): Number of blocks just transferred [default: 1].
            bsize (int, optional): Size of each block (in tqdm units) [default: 1].
            tsize (int, optional): Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if tsize is not None:
            t.total = tsize
        t.update((b - last_b[0]) * bsize)
        last_b[0] = b

    return inner


def web_download(url, local_path, corpus_name='', force_download=False):
    site = request.urlopen(url)
    meta = site.info()
    remote_size = int(meta['Content-Length'])
    if (not force_download) and os.path.exists(local_path) and (os.stat(local_path).st_size == remote_size):
        print(f'[Korpora] Corpus `{corpus_name}` is already installed at {local_path}')
        return None
    filename = os.path.basename(local_path)
    with tqdm(unit='B', unit_scale=True, miniters=1, desc=f'[{corpus_name}] download {filename}') as t:
        request.urlretrieve(url, filename=local_path, reporthook=_reporthook(t))


def web_download_unzip(url, zip_path, corpus_name='', force_download=False):
    web_download(url, zip_path, corpus_name, force_download)
    # assume that path/to/abc.zip consists path/to/abc
    data_path = zip_path[:-4]
    if (not force_download) and os.path.exists(data_path):
        print(f'[Korpora] Corpus `{corpus_name}` is already installed at {data_path}')
        return None
    data_root = os.path.dirname(zip_path)
    with zipfile.ZipFile(zip_path, 'r') as zip_ref:
        zip_ref.extractall(data_root)
    print(f'unzip {data_path}')


def web_download_untar(url, tar_path, corpus_name='', force_download=False):
    web_download(url, tar_path, corpus_name, force_download)
    # assume that path/to/abc.tar consists path/to/abc
    data_path = tar_path[:-4]
    if (not force_download) and os.path.exists(data_path):
        print(f'[Korpora] Corpus `{corpus_name}` is already installed at {data_path}')
        return None
    data_root = os.path.dirname(tar_path)
    with tarfile.open(tar_path) as tar:
        tar.extractall(data_root)
    print(f'decompress {tar_path}')


def web_download_ungzip(url, gzip_path, corpus_name='', force_download=False):
    web_download(url, gzip_path, corpus_name, force_download)
    # assume that path/to/abc.gzip consists path/to/abc
    data_path = gzip_path[:-3]
    if (not force_download) and os.path.exists(data_path):
        print(f'[Korpora] Corpus `{corpus_name}` is already installed at {data_path}')
        return None
    with gzip.open(gzip_path, 'rb') as fi:
        with open(data_path, 'wb') as fo:
            shutil.copyfileobj(fi, fo)
    print(f'decompress {gzip_path}')


def google_drive_download(file_id, local_path, corpus_name='', force_download=False):
    def get_confirm_token(response):
        for key, value in response.cookies.items():
            if key.startswith('download_warning'):
                return value
        return None

    if (not force_download) and os.path.exists(local_path):
        print(f'[Korpora] Corpus `{corpus_name}` is already installed at {local_path}')
        return None

    # init a HTTP session
    session = requests.Session()

    # make a request
    response = session.get(GOOGLE_DRIVE_URL, params={'id': file_id}, stream=True)

    # get confirmation token
    token = get_confirm_token(response)

    if token:
        params = {'id': file_id, 'confirm': token}
        response = session.get(GOOGLE_DRIVE_URL, params=params, stream=True)

    # download to disk
    with open(local_path, "wb") as f:
        content_length = response.headers.get("Content-Length")
        total = int(content_length) if content_length is not None else None
        progress = tqdm(
            unit="B",
            unit_scale=True,
            total=total,
            initial=0,
            desc=f'[{corpus_name}] download {local_path}',
        )
        for chunk in response.iter_content(chunk_size=1024):
            if chunk:
                progress.update(len(chunk))
                f.write(chunk)
        progress.close()


def fetch(remote_path, local_path, corpus_name=None, force_download=False, method="download"):
    """
       Examples::

           >>> from Korpora.utils import fetch
           >>> fetch(
           >>>    'https://raw.githubusercontent.com/e9t/nsmc/master/ratings_train.txt',
           >>>    'nsmc/ratings_train.txt', 'nsmc')
    """
    destination = os.path.abspath(local_path)
    check_dir(destination)

    if method == "download":
        web_download(remote_path, destination, corpus_name, force_download)
    elif method == "google_drive":
        google_drive_download(remote_path, destination, corpus_name, force_download)
    elif method == "download & unzip":
        web_download_unzip(remote_path, destination, corpus_name, force_download)
    elif method == "download & untar":
        web_download_untar(remote_path, destination, corpus_name, force_download)
    elif method == "download & ungzip":
        web_download_ungzip(remote_path, destination, corpus_name, force_download)
    else:
        print(f'download method is not valid ({method})')



import json
import os
import re
from dataclasses import dataclass
from glob import glob
from tqdm import tqdm
from typing import List, Tuple

from .korpora import KorpusData
from .korpus_modu_news import ModuKorpus
from .utils import default_korpora_path


class ModuNEKorpus(ModuKorpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__()
        if root_dir is None:
            root_dir = os.path.join(default_korpora_path, 'NIKL_NE')
        alternative_root_dir = os.path.join(root_dir, 'NIKL_NE')
        if os.path.exists(alternative_root_dir):
            root_dir = alternative_root_dir
        paths = find_corpus_paths(root_dir)
        self.train = KorpusData('모두의_개체명_말뭉치.train', load_modu_ne(paths))
        self.tagmap = {
            'PS': 'PERSON',
            'LC': 'LOCATION',
            'OG': 'ORGANIZATION',
            'AF': 'ARTIFACT',
            'DT': 'DATE',
            'TI': 'TIME',
            'CV': 'CIVILIZATION',
            'AM': 'ANIMAL',
            'PT': 'PLANT',
            'QT': 'QUANTITY',
            'FD': 'STUDY_FIELD',
            'TR': 'THEORY',
            'EV': 'EVENT',
            'MT': 'MATERIAL',
            'TM': 'TERM'
        }


def find_corpus_paths(root_dir_or_paths):
    prefix_pattern = re.compile('[NS]XNE')
    def match(path):
        prefix = path.split(os.path.sep)[-1][:4]
        return prefix_pattern.match(prefix)

    # directory + wildcard
    if isinstance(root_dir_or_paths, str):
        paths = sorted(glob(f'{root_dir_or_paths}/*.json') + glob(root_dir_or_paths))
    else:
        paths = root_dir_or_paths

    paths = [path for path in paths if match(path)]
    if not paths:
        raise ValueError('Not found corpus files. Check `root_dir_or_paths`')
    return paths


@dataclass
class NamedEntityExample:
    sentence_id: str
    sentence: str
    tags: List[str]
    positions: List[Tuple]

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return f"""NamedEntityExample(
    id={self.sentence_id},
    sentence={self.sentence},
    tags={self.tags},
    positions={self.positions}
)"""


def document_to_examples(document):
    examples = []
    sentence = document['sentence']
    for example in sentence:
        example_id = example['id']
        form = example['form']
        tags = [ne['label'] for ne in example['NE']]
        positions = [(ne['begin'], ne['end']) for ne in example['NE']]
        examples.append(NamedEntityExample(example_id, form, tags, positions))
    return examples


def load_modu_ne(paths):
    examples = []
    for path in paths:
        with open(path, encoding='utf-8') as f:
            data = json.load(f)
        documents = data['document']
        desc = f'Loading ModuNE ({os.path.basename(path)})'
        documents_iterator = tqdm(documents, desc=desc, total=len(documents))
        examples += [example for doc in documents_iterator for example in document_to_examples(doc)]
    return examples



import os
from typing import List

from .korpora import Korpus, WordTagKorpusData
from .utils import fetch, default_korpora_path, load_text


NAVER_CHANGWON_NER_FETCH_INFORMATION = [
    {
        'url': 'https://raw.githubusercontent.com/naver/nlp-challenge/master/missions/ner/data/train/train_data',
        'destination': 'naver_changwon_ner/train_data',
        'method': 'download'
    },
]

description = """    Author : 네이버 + 창원대
    Repository : https://github.com/naver/nlp-challenge/tree/master/missions/ner
    References : http://air.changwon.ac.kr/?page_id=10

    개체명(Named Entity)은 인명, 기관명, 지명 등과 같이 문장 또는 문서에서 특정한 의미를 가지고 있는 단어 또는 어구를 말합니다.
    이 때문에 개체명은 정보 검색 및 언어 이해를 위한 분석에서 주요한 대상으로 다루어지고 있습니다.
    Data.ly에서는 개체명 코퍼스를 제공하여 연구에 도움을 드리고자 하며, 공개적인 리더보드를 통해 많은 분들의 연구 동향을 논의/공유하고자 합니다.
    제공되는 코퍼스는 Data.ly에서 제작한 것으로, 연구 및 리더보드를 위한 학습으로 사용 가능하며 상업적인 목적으로 사용될 수 없습니다."""

license = """    연구 및 리더보드를 위한 학습으로 사용 가능하며 상업적인 목적으로 사용될 수 없습니다."""


class NaverChangwonNERKorpus(Korpus):
    def __init__(self, root_dir=None, force_download=False):
        super().__init__(description, license)

        if root_dir is None:
            root_dir = default_korpora_path
        fetch_naverchangwon_ner(root_dir, force_download)

        info = NAVER_CHANGWON_NER_FETCH_INFORMATION[0]
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        self.train = WordTagKorpusData(
            'NaverChangwonNER.train',
            *self.cleaning(load_text(local_path, num_heads=0))
        )

    def cleaning(self, raw_lines: List[str]):
        separated_lines = [line.split('\t') for line in raw_lines]
        all_texts, all_words, all_tags, text, words, tags, = [], [], [], "", [], []
        for separated_line in separated_lines:
            if len(separated_line) == 3:
                _, word, tag = separated_line
                text += word + " "
                words.append(word)
                tags.append(tag)
            else:
                all_texts.append(text)
                all_words.append(words)
                all_tags.append(tags)
                text, words, tags = "", [], []
        return all_texts, all_words, all_tags

    def get_all_words(self):
        return self.train.get_all_words()

    def get_all_tags(self):
        return self.train.get_all_tags()

    def get_all_words_and_tags(self):
        return [item for item in self.train]


def fetch_naverchangwon_ner(root_dir, force_download):
    for info in NAVER_CHANGWON_NER_FETCH_INFORMATION:
        local_path = os.path.join(os.path.abspath(root_dir), info['destination'])
        fetch(info['url'], local_path, 'naver_changwon_ner', force_download)
